/**********************************************************************
** Copyright (C) 2000 Trolltech AS.  All rights reserved.
**
** This file is part of Qt Palmtop Environment.
**
** This file may be distributed and/or modified under the terms of the
** GNU General Public License version 2 as published by the Free Software
** Foundation and appearing in the file LICENSE.GPL included in the
** packaging of this file.
**
** This file is provided AS IS with NO WARRANTY OF ANY KIND, INCLUDING THE
** WARRANTY OF DESIGN, MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE.
**
** See http://www.trolltech.com/gpl/ for GPL licensing information.
**
** Contact info@trolltech.com if any conditions of this licensing are
** not clear to you.
**
**********************************************************************/
#include <qregexp.h>
#include "ablabel.h"
#include "abentry.h"

AbLabel::AbLabel( QWidget *parent, const char *name )
  : QLabel( parent, name ), ent( 0 )
{
}

AbLabel::~AbLabel()
{
}

void AbLabel::init( AbEntry *entry )
{
    ent = entry;
}

void AbLabel::sync()
{
    ASSERT( ent );
    const uint numEdits = 6;
    int fo[numEdits]  = { 4, 8, 3, 1, 0, 6 };

    QString text;
    const QString *value, *company, *state;

    if ( (value = ent->find( "FullName" )) )
	text += "<b>" + *value + "</b><br>";
    if ( (value = ent->find( "Role" )) )
	text += *value + "<br>";

    company =  ent->find( "Company" );
    if ( (value = ent->find( "Department" )) ) {
	text += *value;
	if ( company )
	    text += ", ";
    }
    if (company)
	text += *company + "<br>";

    QStringList fields, trfields;
    fields << "Birthday" << "Email" << "Fax" << "Home Ph" << "Mobile Ph"
	   << "Other" << "Spouse" << "Web Page" << "Work Ph";

    trfields << tr( "Birthday" ) << tr( "Email" ) << tr( "Fax" )
	     << tr( "Home Ph" ) << tr( "Mobile Ph" ) << tr( "Other" )
	     << tr( "Spouse" ) << tr( "Web Page" ) << tr( "Work Ph" );

    const QString *fieldOrder = ent->find( "X-FieldOrder" );
    if (fieldOrder)
	sscanf( fieldOrder->local8Bit(), "%d,%d,%d,%d,%d,%d",
		&fo[0], &fo[1], &fo[2], &fo[3], &fo[4], &fo[5] );

    bool first = TRUE;
    for (uint i = 0; i < numEdits; ++i )
	if ( (value = ent->find( QString( "%1" ).arg( i ))) ) {
	    if ( first )
		text += "<br>";
	    first = FALSE;
	    text += trfields[fo[i]] + ": " + *value + "<br>";
      }


    if ( ent->find( "AddressStreet" ) || ent->find( "AddressCity" ) ||
	 ent->find( "AddressZip" ) || ent->find( "AddressCountry" ) )
	text += "<br>";
    if ( (value = ent->find( "AddressStreet" )) )
	text += *value + "<br>";
    state =  ent->find( "AddressState" );
    if ( (value = ent->find( "AddressCity" )) ) {
	text += *value;
	if ( state )
	    text += ", " + *state;
	text += "<br>";
    }
    else if (state)
	text += *state + "<br>";
    if ( (value = ent->find( "AddressZip" )) )
	text += *value + "<br>";
    if ( (value = ent->find( "AddressCountry" )) )
	text += *value + "<br>";


    if ( (value = ent->find( "Notes" )) )	 {
	QRegExp reg("\n");
	QString tmp = *value;
	tmp.replace( reg, "<br>" );
	text += "<br>" + tmp + "<br>";
    }
    setText( text );
}
