/**********************************************************************
** Copyright (C) 2000 Trolltech AS.  All rights reserved.
**
** This file is part of Qt Palmtop Environment.
**
** This file may be distributed and/or modified under the terms of the
** GNU General Public License version 2 as published by the Free Software
** Foundation and appearing in the file LICENSE.GPL included in the
** packaging of this file.
**
** This file is provided AS IS with NO WARRANTY OF ANY KIND, INCLUDING THE
** WARRANTY OF DESIGN, MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE.
**
** See http://www.trolltech.com/gpl/ for GPL licensing information.
**
** Contact info@trolltech.com if any conditions of this licensing are
** not clear to you.
**
**********************************************************************/
#include "ablineedit.h"
#include "abentry.h"

/*!
  \class AbLineEdit ablineedit.h

  \brief QLineEdit based class with a purpose hint.
*/

AbLineEdit::AbLineEdit( QWidget *parent, const char *name )
    : QLineEdit( parent, name ), isHinting( TRUE ), ent( 0 )
{
}

AbLineEdit::~AbLineEdit()
{
}

void AbLineEdit::init( AbEntry *entry, char *name, 
		       const QString & hint )
{
    purposeHintText = hint;
    if ( isHinting )
        setText( purposeHintText );
    
    ent = entry;
    setName( name );
}

void AbLineEdit::setName( char *name )
{
    QLineEdit::setName( name );
    sync();
}

void AbLineEdit::sync()
{
    ASSERT( ent );

    const QString *value = ent->find( name() );
    if ( value ) {
        isHinting = FALSE;
        if ( *value != text() )
	    setText( *value );
    } else {
        isHinting = TRUE;
        setText( purposeHintText );    
    }
}

void AbLineEdit::focusInEvent( QFocusEvent * e )
{
    if ( isHinting ) {
        setText( "" );
        isHinting = FALSE;
    }
    QLineEdit::focusInEvent( e );
}

void AbLineEdit::focusOutEvent( QFocusEvent * e )
{
    ASSERT( ent );
    ent->replace( QString( name()), new QString( text()) );

    if ( text().isEmpty() ) {
        ent->remove( QString( name () ));
        isHinting = TRUE;
        setText( purposeHintText );
    }
    QLineEdit::focusOutEvent( e );
}


