/**********************************************************************
** Copyright (C) 2000 Trolltech AS.  All rights reserved.
**
** This file is part of Qt Palmtop Environment.
**
** This file may be distributed and/or modified under the terms of the
** GNU General Public License version 2 as published by the Free Software
** Foundation and appearing in the file LICENSE.GPL included in the
** packaging of this file.
**
** This file is provided AS IS with NO WARRANTY OF ANY KIND, INCLUDING THE
** WARRANTY OF DESIGN, MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE.
**
** See http://www.trolltech.com/gpl/ for GPL licensing information.
**
** Contact info@trolltech.com if any conditions of this licensing are
** not clear to you.
**
**********************************************************************/
#ifndef CALCULATORIMPL_H
#define CALCULATORIMPL_H

#include <qlcdnumber.h>
#include "calculator.h"

class QLabel;
class CalculatorImpl : public Calculator
{
    Q_OBJECT
    
public:
    CalculatorImpl( QWidget * parent = 0, const char * name = 0, 
		    WFlags f = 0 );
    
public slots:
    void digit_0(){ enterNumber( 0 ); }
    void digit_1(){ enterNumber( 1 ); }
    void digit_2(){ enterNumber( 2 ); }
    void digit_3(){ enterNumber( 3 ); }
    void digit_4(){ enterNumber( 4 ); }
    void digit_5(){ enterNumber( 5 ); }
    void digit_6(){ enterNumber( 6 ); }
    void digit_7(){ enterNumber( 7 ); }
    void digit_8(){ enterNumber( 8 ); }
    void digit_9(){ enterNumber( 9 ); }

    void memClear();
    void memRecall();
    void memAdd();
    
    void percent(){ execOp( oPercent ); };
    void divX(){ execOp( oDivX ); };
    void xSquared(){ execOp( oXsquared ); };
    void point(){ execOp( oPoint ); };
    void changeSign(){ execOp( oChSign ); };
    void sum(){ execOp( oSum ); }
    void subtract(){ execOp( oSub); }
    void multiply(){ execOp( oMult ); }
    void add()     { execOp( oAdd ); }
    void divide()  { execOp( oDiv ); }
    
    void clear(){ acc = num = 0; lastOp = lastAc = oNop; 
                  state = sStart; numDecimals = 0; flPoint = FALSE; 
		  lcd->display( 0 ); }

private:
    enum Operations { oNop, oChSign, oPoint, oSum, oAdd, oSub, oDiv, oMult,
                      oPercent, oDivX, oXsquared };
    enum States { sStart, sNewNumber, sContinue };

    Operations lastOp, lastAc;
    States     state;
    
    double acc, num, mem;
    int    numDecimals;
    bool   flPoint;
    
    void   enterNumber( int n );
    void   execOp( Operations i );
    double evalExpr( Operations op );
    QLabel * memMark;
}; 

#endif
