/**********************************************************************
** Copyright (C) 2000 Trolltech AS.  All rights reserved.
**
** This file is part of Qt Palmtop Environment.
**
** This file may be distributed and/or modified under the terms of the
** GNU General Public License version 2 as published by the Free Software
** Foundation and appearing in the file LICENSE.GPL included in the
** packaging of this file.
**
** This file is provided AS IS with NO WARRANTY OF ANY KIND, INCLUDING THE
** WARRANTY OF DESIGN, MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE.
**
** See http://www.trolltech.com/gpl/ for GPL licensing information.
**
** Contact info@trolltech.com if any conditions of this licensing are
** not clear to you.
**
**********************************************************************/


#include <time.h>
#include <qlayout.h>
#include <qlabel.h>
#include <qspinbox.h>
#include <qlabel.h>
#include "datebookmonth.h"
#include "settime.h"


SetDateTime::SetDateTime(QWidget *parent, const char *name, bool modal,  WFlags f )
    : QDialog( parent, name, modal, f )
{
    setCaption( tr("Set System Time") );

    QVBoxLayout *vb = new QVBoxLayout( this, 5 );

    //QLabel *l = new QLabel( tr("<b>Time</b>"), this );
    //vb->addWidget( l );

    time = new SetTime( this );
    vb->addWidget( time );

    QFrame *f = new QFrame( this );
    f->setFrameStyle( QFrame::HLine | QFrame::Sunken );
    vb->addWidget( f );

    //l = new QLabel( tr("<b>Date</b>"), this );
    //vb->addWidget( l );

    date = new SetDate( this );
    vb->addWidget( date );
}

void SetDateTime::accept()
{
    QDateTime dt( date->date(), time->time() );
    QDateTime now = QDateTime::currentDateTime();
    int change = now.secsTo( dt );

    time_t t = ::time(0);
    t += change;
    stime(&t);

    QDialog::accept();
}

SetTime::SetTime( QWidget *parent, const char *name )
    : QWidget( parent, name )
{
    QTime currTime = QTime::currentTime();
    hour = currTime.hour();
    minute = currTime.minute();

    QHBoxLayout *hb = new QHBoxLayout( this );

    QLabel *l = new QLabel( tr("Hour"), this );
    l->setAlignment( AlignRight | AlignVCenter );
    hb->addWidget( l );

    QSpinBox *sb = new QSpinBox( this );
    sb->setMinValue( 0 );
    sb->setMaxValue( 23 );
    sb->setValue( hour );
    connect( sb, SIGNAL(valueChanged(int)), this, SLOT(hourChanged(int)) );
    hb->addWidget( sb );

    l = new QLabel( tr("Minute"), this );
    l->setAlignment( AlignRight | AlignVCenter );
    hb->addWidget( l );

    sb = new QSpinBox( this );
    sb->setMinValue( 0 );
    sb->setMaxValue( 59 );
    sb->setValue( minute );
    connect( sb, SIGNAL(valueChanged(int)), this, SLOT(minuteChanged(int)) );
    hb->addWidget( sb );
}

QTime SetTime::time() const
{
    return QTime( hour, minute, 0 );
}

void SetTime::hourChanged( int value )
{
    hour = value;
}

void SetTime::minuteChanged( int value )
{
    minute = value;
}


SetDate::SetDate( QWidget *parent, const char *name )
    : QWidget( parent, name )
{
    m_date = QDate::currentDate();

    QHBoxLayout *hb = new QHBoxLayout( this );

    DateBookMonth *dbm = new DateBookMonth( this );
    
    connect( dbm, SIGNAL(dateClicked(int,int,int)),
	     this, SLOT(dateClicked(int,int,int)) );
    hb->addWidget( dbm );
}

void SetDate::dateClicked( int year, int month, int day )
{
    m_date.setYMD( year, month, day );
}

