/**********************************************************************
** Copyright (C) 2000 Trolltech AS.  All rights reserved.
**
** This file is part of Qt Palmtop Environment.
**
** This file may be distributed and/or modified under the terms of the
** GNU General Public License version 2 as published by the Free Software
** Foundation and appearing in the file LICENSE.GPL included in the
** packaging of this file.
**
** This file is provided AS IS with NO WARRANTY OF ANY KIND, INCLUDING THE
** WARRANTY OF DESIGN, MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE.
**
** See http://www.trolltech.com/gpl/ for GPL licensing information.
**
** Contact info@trolltech.com if any conditions of this licensing are
** not clear to you.
**
**********************************************************************/

#include "qimpenchar.h"
#include "qimpencharedit.h"

#define ID_STROKEPOINT	10

class StrokePoint : public QCanvasRectangle
{
public:
    StrokePoint( int x, int y, int w, int h, QCanvas *c )
     : QCanvasRectangle( x, y, w, h, c ) 
    {
	setBrush( black );
    }

    virtual int rtti() const { return ID_STROKEPOINT; }
};

QIMPenCharEditor::QIMPenCharEditor( QWidget *parent, const char *name, WFlags f )
 : QCanvasView( new QCanvas(700, 700), parent, name, f )
{
    scale = 10;
    pntSize = 5;
    moveItem = 0;
}

void QIMPenCharEditor::setCharacter( QIMPenChar *c )
{
    ch = c;
    makeStrokes();
}

void QIMPenCharEditor::makeStrokes()
{
    strokes.clear();

    const QList<QIMPenStroke> &penStrokes = ch->penStrokes();
    QListIterator<QIMPenStroke> it( penStrokes );
    for ( ; it.current(); ++it ) {
	//qDebug( "Adding stroke" );
	ItemList *str = new ItemList;
	strokes.append( str );
	const QArray<QIMPenGlyphLink> &chain = it.current()->chain();
	QPoint p = it.current()->startingPoint();
	QRect br( it.current()->boundingRect() );
	p.setX( (75 - br.width()) / 2 + (p.x () - br.left()) );
	int x = p.x() * scale;
	int y = p.y() * scale;
	StrokePoint *sp = new StrokePoint( x - pntSize/2, y - pntSize/2, pntSize, pntSize, canvas() );
	sp->show();
	str->append( sp );
	for ( unsigned i = 0; i < chain.count(); i++ ) {
	    QIMPenGlyphLink l = chain[i];
	    int nx = x + l.dx * scale;
	    int ny = y + l.dy * scale;
	    QCanvasLine *cl = new QCanvasLine( canvas() );
	    cl->setPoints( x, y, nx, ny );
	    cl->show();
	    str->append( cl );
	    sp = new StrokePoint( nx - pntSize/2, ny - pntSize/2, pntSize, pntSize, canvas() );
	    sp->show();
	    str->append( sp );
	    x = nx;
	    y = ny;
	}
    }

    canvas()->update();
}

void QIMPenCharEditor::movePoint( StrokePoint *p, int x, int y )
{
    int curr = -1;
    ItemList *l = 0;
    for ( l = strokes.first(); l; l = strokes.next() ) {
	if ( ( curr = l->findRef( p ) ) != -1 )
	    break;
    }
    if ( !l )
	return;

    p->move( (int)(x - pntSize/2), (int)(y - pntSize/2) );

    if ( curr > 0 ) {
	QCanvasLine *li = (QCanvasLine *)l->at(curr-1);
	li->setPoints( li->startPoint().x(), li->startPoint().y(), x, y );
    }
    if ( curr < l->count() ) {
	QCanvasLine *li = (QCanvasLine *)l->at(curr+1);
	li->setPoints( x, y, li->endPoint().x(), li->endPoint().y() );
    }
}

void QIMPenCharEditor::contentsMousePressEvent(QMouseEvent* e)
{
    QCanvasItemList list = canvas()->collisions(e->pos());
    QCanvasItemList::Iterator it;
    for ( it = list.begin(); it != list.end(); ++it ) {
	if ( (*it)->rtti() == ID_STROKEPOINT )
	    break;
    }
    if ( it != list.end() )
	moveItem = (StrokePoint *) (*it);
}

void QIMPenCharEditor::contentsMouseMoveEvent(QMouseEvent* e)
{
    if ( moveItem ) {
	QPoint np = e->pos() / scale;
	np *= scale;
	QPoint cp( (int)moveItem->x(), (int)moveItem->y() );
	if ( np != cp ) {
	    movePoint( moveItem, np.x(), np.y() );
	    canvas()->update();
	}
    }
}

void QIMPenCharEditor::contentsMouseReleaseEvent(QMouseEvent* e)
{
    moveItem = 0;
}




