/**********************************************************************
** Copyright (C) 2000 Trolltech AS.  All rights reserved.
**
** This file is part of Qt Palmtop Environment.
**
** This file may be distributed and/or modified under the terms of the
** GNU General Public License version 2 as published by the Free Software
** Foundation and appearing in the file LICENSE.GPL included in the
** packaging of this file.
**
** This file is provided AS IS with NO WARRANTY OF ANY KIND, INCLUDING THE
** WARRANTY OF DESIGN, MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE.
**
** See http://www.trolltech.com/gpl/ for GPL licensing information.
**
** Contact info@trolltech.com if any conditions of this licensing are
** not clear to you.
**
**********************************************************************/
#ifndef ORGANIZER_H
#define ORGANIZER_H

#include <qvaluelist.h>
#include <qvector.h>
#include <qdatetime.h>
#include <qpixmap.h>
#include <qstringlist.h>
#include <qtextstream.h>

#include "xmlreader.h"

class OrganizerBase
{
public:
    OrganizerBase();
    int uid() const;

private:
    int id;

};

class Note
{
public:
    Note();

    void setText( const QString &text );
    void setPixmap( const QPixmap &pix );
    QString text() const;
    QPixmap pixmap() const;

private:
    QPixmap pix;
    QString txt;

};

class Todo : public OrganizerBase
{
public:
    Todo();

    void setDate( bool b );
    void setDate( const QDate &dt );
    bool hasDate() const;
    QDate date() const;

    void setCompleted( bool b );
    bool isCompleted() const;

    void setPriority( int p );
    int priority() const;

    void setDescription( const QString &d );
    QString description() const;

    void setCategory( const QString &c );
    QString category() const;

    void setNotes( const Note &n );
    Note notes() const;

    void save( QTextStream &ts, const QString &indent );
    void load( Node *n );

private:
    bool hDate;
    QDate dt;
    bool complete;
    int prior;
    QString descript, categ;
    Note note;

};

class Event
{
public:
    struct RepeatPattern
    {
	bool something;
	bool operator==( const RepeatPattern & ) const { return TRUE; }
    };

    Event();

    enum Type { Normal, AllDay };

    void setDescription( const QString &s );
    void setLocation( const QString &s );
    void setCategory( const QString &s );
    void setType( Type t );
    void setStart( const QDateTime &d );
    void setEnd( const QDateTime &e );
    void setAlarm( bool b, int minutes );
    void setRepeat( bool b, const RepeatPattern &p );
    void setNotes( const QString &n );

    QString description() const;
    QString location() const;
    QString category() const;
    Type type() const;
    QDateTime start( bool actual = FALSE ) const;
    QDateTime end( bool actual = FALSE ) const;
    bool hasAlarm() const;
    int alarmTime() const;
    bool doRepeat() const;
    RepeatPattern repeatPattern() const;
    QString notes() const;

    bool operator==( const Event &e ) const;

    void save( QTextStream &ts, const QString &indent );
    void load( Node *n );

private:
    QString descript, locat, categ;
    Type typ;
    QDateTime startDate, endDate;
    bool hAlarm, hRepeat;
    int aMinutes;
    RepeatPattern pattern;
    QString note;

};

#endif
