#include <qmainwindow.h>
#include <qstatusbar.h>
#include <qmessagebox.h>
#include <qmenubar.h>
#include <qapplication.h>
#include <qprogressdialog.h>
#include "mp3player.h"
#include "mediabutton.h"

#include "fileselector.h"
#include "mpegplayer.h"
#include "mpegview.h"
#include "mpegcontrol.h"
#include "pixmaploader.h"

#include <qtoolbar.h>
#include <qwidgetstack.h>
#include <qpopupmenu.h>
#include <qlayout.h>
 


Mp3PlayerWindow::Mp3PlayerWindow(QWidget* parent, const char* name, WFlags f) :
    QMainWindow(parent,name,f)
{
    setFixedSize( 240, 320 );

    QMenuBar *menu = menuBar();
    QPopupMenu *file = new QPopupMenu( this );
    file->insertItem("&Open Mp3", this, SLOT(openMp3()));
    file->insertItem("Open &Playlist", this, SLOT(openPlaylist()));
    file->insertSeparator();
    file->insertItem("&Close", this, SLOT(close()));
    menu->insertItem("&File", file);
    menu->insertSeparator();
    QPopupMenu *settings = new QPopupMenu( this );
    settings->insertItem("&Options", this, SLOT(options()));
    settings->insertItem("&Edit Playlist", this, SLOT(editPlaylist()));
    menu->insertItem("&Settings", settings);
    menu->insertSeparator();
    QPopupMenu *help = new QPopupMenu( this );
    help->insertItem("&About", this, SLOT(about()));
    menu->insertItem("&Help", help);


    stack = new QWidgetStack(this);
    setCentralWidget( stack );

    controls = new QFrame( stack );
    mpeg = new MpegView( stack );
    
    controls->setBackgroundPixmap( PixmapLoader::loadPixmap( "metalFinish" ) );

    int yo = -2; // 12;
    int xo = -2;
    
    playButton			= new MediaButton( controls, TRUE,  3*30-15+xo,3*30-13+yo, 0, 1 );
//    connect( playButton, SIGNAL( pressed() ), mpeg, SLOT( playAgain() ) );
//    connect( playButton, SIGNAL( toggled() ), mpeg, SLOT( stop() ) );
    connect( playButton, SIGNAL( toggled( bool ) ), this, SLOT( playToggle( bool ) ) );

    MediaButton *backButton	= new MediaButton( controls, FALSE, 0*30+5+xo, 3*30+yo,    4 );
    Q_UNUSED( backButton );
    
    MediaButton *playlistButton	= new MediaButton( controls, FALSE, 1*30+xo,   1*30+yo,    8 );
    Q_UNUSED( playlistButton );
    
    MediaButton *upButton	= new MediaButton( controls, FALSE, 3*30+xo,   0*30+5+yo,  9 ); // 5
    connect( upButton, SIGNAL( pressed() ), this, SLOT( incVolume() ) );
    
    MediaButton *loopButton	= new MediaButton( controls, TRUE,  5*30+xo,   1*30+yo,    7 );
    connect( loopButton, SIGNAL( toggled(bool) ), mpeg, SLOT( setLooping(bool) ) );
    
    MediaButton *forwardButton	= new MediaButton( controls, FALSE, 6*30-5+xo, 3*30+yo,    3 );
    Q_UNUSED( forwardButton );
    
    MediaButton *pauseButton	= new MediaButton( controls, TRUE,  5*30+xo,   5*30+yo,    2 );
//    connect( pauseButton, SIGNAL( clicked() ), pauseButton, SLOT( toggle() ) );
    Q_UNUSED( pauseButton );

    MediaButton *downButton	= new MediaButton( controls, FALSE, 3*30+xo,   6*30-5+yo,  10 ); // 6
    connect( downButton, SIGNAL( pressed() ), this, SLOT( decVolume() ) );
    
    MediaButton *stopButton	= new MediaButton( controls, FALSE, 1*30+xo,   5*30+yo,    1 );
    connect( stopButton, SIGNAL( clicked() ), mpeg, SLOT( stop() ) );
    connect( stopButton, SIGNAL( clicked() ), playButton, SLOT( toggle() ) );

    fileSelector = new FileSelector("*.mp3", stack, "fs");
    connect( fileSelector, SIGNAL( closeMe() ), this, SLOT( closeFileSelector() ) );
    connect( fileSelector, SIGNAL( fileSelected( const FileManager::File &) ), this, SLOT( openFile( const FileManager::File & ) ) );
    
    stack->raiseWidget( controls );

    // Might retrieve volume from some user setting from
    // last time or something
    volume = 50;
    mpeg->setVolume( volume );

    // statusBar();

    init();
}


Mp3PlayerWindow::~Mp3PlayerWindow()
{
    delete mpeg;
}


void Mp3PlayerWindow::openMp3()
{
    open();
}


void Mp3PlayerWindow::openPlaylist()
{
}


void Mp3PlayerWindow::options()
{
}


void Mp3PlayerWindow::editPlaylist()
{
}


void Mp3PlayerWindow::init()
{
}


void Mp3PlayerWindow::about()
{
    static QMessageBox* about = new QMessageBox( "Qt Mp3 Player",
	    "<center><h3>Qt Mp3 Player<p>for handhelds</h3>"
	    "Version 1.0.0.0000"
	    "<p>Copyright (C) 2000 Trolltech",
	    QMessageBox::NoIcon, 1, 0, 0, this, 0, FALSE );
    about->setButtonText( 1, "Ok" );
    about->show();
}


void Mp3PlayerWindow::play(const QString& filename)
{
    mpeg->play( filename );
}


void Mp3PlayerWindow::openFile( const FileManager::File &file )
{
    FileManager fm;
    play( fm.absoluteFileName( file ) );
}


void Mp3PlayerWindow::open()
{
    stack->raiseWidget( fileSelector );
}


void Mp3PlayerWindow::closeFileSelector()
{
    stack->raiseWidget( controls );
}                                                                                         


void Mp3PlayerWindow::incVolume()
{
    if (volume < 100)
        volume++;
    mpeg->setVolume( volume );
}                                                                                         


void Mp3PlayerWindow::decVolume()
{
    if (volume > 0)
        volume--;
    mpeg->setVolume( volume );
}                                                                                         


void Mp3PlayerWindow::playToggle( bool on )
{
    if ( on ) {
	playButton->setControlType( 0 );
	mpeg->playAgain();
    } else {
	playButton->setControlType( 2 );
    }
}


