/**********************************************************************
** Copyright (C) 2000 Trolltech AS.  All rights reserved.
**
** This file is part of Qt Palmtop Environment.
**
** This file may be distributed and/or modified under the terms of the
** GNU General Public License version 2 as published by the Free Software
** Foundation and appearing in the file LICENSE.GPL included in the
** packaging of this file.
**
** This file is provided AS IS with NO WARRANTY OF ANY KIND, INCLUDING THE
** WARRANTY OF DESIGN, MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE.
**
** See http://www.trolltech.com/gpl/ for GPL licensing information.
**
** Contact info@trolltech.com if any conditions of this licensing are
** not clear to you.
**
**********************************************************************/
#include "mpegplayer.h"
#include "mpegview.h"
#include "mpegcontrol.h"
#include "fileselector.h"
#include "pixmaploader.h"


#include <qslider.h>
#include <qtoolbar.h>
#include <qwidgetstack.h>
#include <qapplication.h>
#include <qpopupmenu.h>
#include <qmenubar.h>
#include <qlayout.h>

#include <qaction.h>

#define MPEG_NO_CONTROL


MpegPlayer::MpegPlayer( QWidget* parent, const char* name, WFlags fl ) :
    QMainWindow(parent,name,fl)
{
    setToolBarsMovable( FALSE );
    setCaption( tr("MPEG Player") );
    
    QToolBar *bar = new QToolBar( this );
    bar->setHorizontalStretchable( TRUE );

    stack = new QWidgetStack(this);
    setCentralWidget(stack);

    mpegpanel = new QWidget(stack);

    mpeg = new MpegView(mpegpanel);
    connect(mpeg, SIGNAL(clicked()), this, SLOT(normalView()));
    mpeg->setBackgroundColor(black);

#ifdef MPEG_NO_CONTROL
    control = 0;
#else    
    control = new MpegControl(mpegpanel);
    connect(mpeg, SIGNAL(frameDone(long)),
	    control, SLOT(setCurrentFrame(long)));
    connect(control, SIGNAL(currentFrameChanged(long)),
	    mpeg, SLOT(setCurrentFrame(long)));
#endif
    vb=0; relayout();

    fileSelector = new FileSelector("*.mpeg;*.mpg;*.m2v;*.mp3", stack, "fs");
    connect( fileSelector, SIGNAL( closeMe() ), this, SLOT( closeFileSelector() ) );
    connect( fileSelector, SIGNAL( fileSelected( const FileManager::File &) ), this, SLOT( openFile( const FileManager::File & ) ) );


        QAction * a = new QAction( tr( "Open" ), PixmapLoader::loadPixmap( "fileopen" ), QString::null, CTRL + Key_O, this, 0 );
    connect( a, SIGNAL( activated() ), this, SLOT( open() ) );
    a->addTo( bar );

    a = new QAction( tr( "Fullscreen" ), PixmapLoader::loadPixmap( "fullscreen" ), QString::null, 0, this, 0 );
    connect( a, SIGNAL( activated() ), this, SLOT( fullScreen() ) );
    a->addTo( bar );


    QSlider *sl = new QSlider( Horizontal, bar );
    connect(sl, SIGNAL(valueChanged(int)), mpeg, SLOT(setVolume(int)));
    sl->setValue(66);
    sl->setFixedWidth(50);
    
    a = new QAction( tr( "Mute" ), PixmapLoader::loadPixmap( "mute" ), 
		     QString::null, 0, this, 0, TRUE );
    connect( a, SIGNAL( toggled(bool) ), mpeg, SLOT( setMute(bool) ) );
    a->addTo( bar );

    
    a = new QAction( tr( "Play" ), PixmapLoader::loadPixmap( "play" ), QString::null, 0, this, 0 );
    connect( a, SIGNAL( activated() ), mpeg, SLOT( playAgain() ) );
    a->addTo( bar );

    a = new QAction( tr( "Stop" ), PixmapLoader::loadPixmap( "stop" ), QString::null, 0, this, 0 );
    connect( a, SIGNAL( activated() ), mpeg, SLOT( stop() ) );
    a->addTo( bar );

    a = new QAction( tr( "Loop" ), PixmapLoader::loadPixmap( "repeat" ), 
		     QString::null, 0, this, 0, TRUE );
    connect( a, SIGNAL( toggled(bool) ), mpeg, SLOT( setLooping(bool) ) );
    a->addTo( bar );
    
    open();
}

MpegPlayer::~MpegPlayer()
{
    delete mpeg; // in case it is fullscreen (reparented)
}


void MpegPlayer::relayout()
{
    delete vb;
    vb = new QVBoxLayout(mpegpanel);
    vb->addWidget(mpeg);
#ifndef MPEG_NO_CONTROL
    vb->addWidget(control);
#endif
}

bool MpegPlayer::play(const QString& filename)
{
    if ( mpeg->play(filename) ) {
#ifndef MPEG_NO_CONTROL
	control->setCurrentFrame(0);
	control->setTotalFrames(mpeg->totalFrames());
#endif
        stack->raiseWidget(mpegpanel);
	return TRUE;
    }
    return FALSE;
}

void MpegPlayer::openFile( const FileManager::File &file )
{
    FileManager fm;
    setCaption( tr("MPEG Player") + " - " + file.filename );
    play(fm.absoluteFileName(file));
}

void MpegPlayer::open()
{
    stack->raiseWidget(fileSelector);
}

void MpegPlayer::closeFileSelector()
{
    stack->raiseWidget(mpegpanel);
}


void MpegPlayer::normalView()
{
    if ( !mpeg->parentWidget() ) {
	mpeg->reparent(mpegpanel,0,QPoint(0,0),FALSE);
	relayout();
	mpeg->resize(mpeg->sizeHint());
	mpeg->show();
    }
}

void MpegPlayer::fullScreen()
{
    if ( mpeg->parentWidget() ) {
	mpeg->reparent(0,WStyle_Tool | WStyle_Customize | WStyle_StaysOnTop,
	    QPoint(0,0),FALSE);
	mpeg->resize(qApp->desktop()->width(), qApp->desktop()->height());
	mpeg->show();
    }
}

