/**********************************************************************
** Copyright (C) 2000 Trolltech AS.  All rights reserved.
**
** This file is part of Qt Palmtop Environment.
**
** This file may be distributed and/or modified under the terms of the
** GNU General Public License version 2 as published by the Free Software
** Foundation and appearing in the file LICENSE.GPL included in the
** packaging of this file.
**
** This file is provided AS IS with NO WARRANTY OF ANY KIND, INCLUDING THE
** WARRANTY OF DESIGN, MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE.
**
** See http://www.trolltech.com/gpl/ for GPL licensing information.
**
** Contact info@trolltech.com if any conditions of this licensing are
** not clear to you.
**
**********************************************************************/
#include "settings.h"

#include <qlabel.h>
#include <qcheckbox.h>
#include <qradiobutton.h>
#include <qtabwidget.h>
#include <qslider.h>
#include <qfile.h>
#include <qtextstream.h>
#include <qmessagebox.h>
#ifdef QT_QWS_IPAQ
#define u_char uchar // junk in linux/h3600_ts.h
#define u_short ushort // junk in linux/h3600_ts.h
#include <linux/h3600_ts.h>
#include <unistd.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <fcntl.h>
#include <sys/ioctl.h>
#endif
#include <stdlib.h>

Settings::Settings( QWidget* parent,  const char* name, WFlags fl )
    : SettingsBase( parent, name, TRUE, fl )
{
    reset();
#ifdef SINGLE_APP
    rotation_message->setText(
	"<p>Rotation will not apply until you restart the server");
#endif

    connect(brightness, SIGNAL(valueChanged(int)), this, SLOT(applyBrightness()));

    // Not supported
    auto_brightness->hide();
    auto_volume->hide();

#ifdef QT_QWS_CASSIOPEIA
    tabWidget->removePage( Widget2 );
    tabWidget->removePage( tab );
#endif
}

Settings::~Settings()
{
}

void Settings::accept()
{
    applyBrightness();
    applyRotation();
    QDialog::accept();
}

static bool powersaving=FALSE;
static int bright=100;
static bool saver_on=TRUE;

static void set_fl()
{
#ifdef QT_QWS_IPAQ
    QString cmd = "/usr/bin/bl 1 ";
    cmd += bright<=0 || saver_on && powersaving ? "0 " : "1 ";
    cmd += QString::number(bright);
    system(cmd.latin1());
#endif
}

void Settings::setPowerSaving(bool on)
{
    powersaving = on;
    set_fl();
}

void Settings::applyBrightness()
{
    saver_on = screensaver->isChecked();
    bright = 255-brightness->value();
    set_fl();
}

static int initrot=-1;

void Settings::applyRotation()
{
    if ( rot270->isChecked() )
	setRotation( 270 );
    else if ( rot180->isChecked() )
	setRotation( 180 );
    else if ( rot90->isChecked() )
	setRotation( 90 );
    else if ( rot0->isChecked() )
	setRotation( 0 );
}

void Settings::setRotation(int r)
{
    QString rs;
    int sr = (r+initrot)%360;
    if ( sr ) {
        rs = "Transformed:Rot";
	rs += QString::number(sr);
    } else {
        rs = "LinuxFb";
    }
    setenv("QWS_DISPLAY", rs.latin1(), 1);

#if !defined(QT_QWS_IPAQ) && !defined(QT_QWS_CASSIOPEIA)
    // save the rotation configuration to a file
    QFile f(".rotation.conf");
    
    if (f.open(IO_WriteOnly)) {
	QTextStream ts(&f);
	ts << rs;
	f.close();
    } else {
	QMessageBox::warning( 0, "Warning", "Could not save rotation settings to .rotation.conf", "Ok", 0, 0, 0, 1 );
    }
#endif
}

void Settings::reject()
{
    reset();
    QDialog::reject();
}

void Settings::reset()
{
    QString d = getenv("QWS_DISPLAY");
    int r=0;
    if ( d.contains("Rot90") ) {
	r = 90;
    } else if ( d.contains("Rot180") ) {
	r = 180;
    } else if ( d.contains("Rot270") ) {
	r = 270;
    }
    if ( initrot == -1 )
	initrot = r;
    r = (r-initrot+360) % 360;
    rot0->setChecked(r==0);
    rot90->setChecked(r==90);
    rot180->setChecked(r==180);
    rot270->setChecked(r==270);
    brightness->setValue(128); // #### Not stored...
}
