/**********************************************************************
** Copyright (C) 2000 Trolltech AS.  All rights reserved.
**
** This file is part of Qt Palmtop Environment.
**
** This file may be distributed and/or modified under the terms of the
** GNU General Public License version 2 as published by the Free Software
** Foundation and appearing in the file LICENSE.GPL included in the
** packaging of this file.
**
** This file is provided AS IS with NO WARRANTY OF ANY KIND, INCLUDING THE
** WARRANTY OF DESIGN, MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE.
**
** See http://www.trolltech.com/gpl/ for GPL licensing information.
**
** Contact info@trolltech.com if any conditions of this licensing are
** not clear to you.
**
**********************************************************************/
#ifndef __CANVAS_CARD_H__
#define __CANVAS_CARD_H__


#include <qpainter.h>
#include <qimage.h>
#include <qpaintdevice.h>
#include <qcanvas.h>
#include "cardgame.h"


#ifdef _WS_QWS_
# include "pixmaploader.h"
# define findPixmap( filename ) \
    PixmapLoader::findPixmap( filename )
#else 
# define PICS_DIR    "../pics/"
# define findPixmap( filename ) \
    PICS_DIR filename ".png"
#endif


// ### Just made the number up, is that what you do???
static const int canvasCardId = 2434321;


class CanvasCard : public Card, public QCanvasRectangle // QCanvasSprite
{
public:
    CanvasCard( eValue v, eSuit s, bool f, QCanvas *canvas ) :
	Card(v, s, f), QCanvasRectangle( 0, 0, 1, 1, canvas ), cardBack(1), scaleX(1.0), scaleY(1.0) {
	if (!getPixmapSize)
	    getPixmapSize = new QPixmap( findPixmap( "card_face" ) );
	xOff = getPixmapSize->width() / 2;
	yOff = getPixmapSize->height() / 2;
	setSize(getPixmapSize->width(), getPixmapSize->height());
	setPen(NoPen);
	flipping = FALSE;
    }
    virtual ~CanvasCard() { canvas()->removeItem(this); }

    int rtti () const { return canvasCardId; }
    void move(QPoint p) { QCanvasItem::move( p.x(), p.y() ); }
    void move(int x, int y) { QCanvasItem::move( x, y ); }
    void animatedMove(int x, int y, int steps = 10);
    void animatedMove() { animatedMove(savedX, savedY); }
    void savePos(void) { savedX = (int)x(); savedY = (int)y(); }
    void moveToPile(int p) { Q_UNUSED(p); }
    void setCardBack(int b) { cardBack = b; if (!isFacing()) redraw(); }
    
    virtual void flipTo(int x, int y, int steps = 8);
    virtual void setPos( int x, int y, int z ) { setX( x ); setY( y ); setZ( z ); }
    virtual void showCard(void) { show(); }
    virtual void redraw(void) { hide(); show(); }

    void advance(int stage);

protected:
    virtual void flip(void) { redraw(); }
    virtual void draw(QPainter &p);

private:
    int destX, destY;
    int animSteps;
    int flipSteps;
    bool flipping;
    int savedX, savedY;
    int cardBack;
    int oldCardBack;
    double scaleX, scaleY;
    int xOff, yOff;
    static QPixmap *getPixmapSize;
};


#endif

