/**********************************************************************
** Copyright (C) 2000 Trolltech AS.  All rights reserved.
**
** This file is part of Qt Palmtop Environment.
**
** This file may be distributed and/or modified under the terms of the
** GNU General Public License version 2 as published by the Free Software
** Foundation and appearing in the file LICENSE.GPL included in the
** packaging of this file.
**
** This file is provided AS IS with NO WARRANTY OF ANY KIND, INCLUDING THE
** WARRANTY OF DESIGN, MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE.
**
** See http://www.trolltech.com/gpl/ for GPL licensing information.
**
** Contact info@trolltech.com if any conditions of this licensing are
** not clear to you.
**
**********************************************************************/

#include <stdio.h>
#include <qlayout.h>
#include <qlabel.h>
#include <qpainter.h>
#include <qpixmap.h>
#include <qtimer.h>
#include "load.h"

LoadInfo::LoadInfo( QWidget *parent, const char *name, WFlags f )
    : QWidget( parent, name, f )
{
    QVBoxLayout *vb = new QVBoxLayout( this, 5 );

    vb->addWidget( new Load( this ), 2 );
    QLabel *l = new QLabel( this );
    l->setPixmap( makeLabel( red, "User usage (%)" ) );
    vb->addWidget( l, 1 );
    l = new QLabel( this );
    l->setPixmap( makeLabel( green, "System usage (%)" ) );
    vb->addWidget( l, 1 );
//    vb->addStretch( 1 );
}

QPixmap LoadInfo::makeLabel( const QColor &col, const QString &text )
{
    int h = fontMetrics().height();
    QPixmap pm( 20 + fontMetrics().width( text ), h );
    QPainter p( &pm );
    p.fillRect( pm.rect(), colorGroup().background() );
    p.setPen( col );
    p.drawLine( 0, h/2, 15, h/2 );
    p.setPen( colorGroup().text() );
    p.drawText( 20, fontMetrics().ascent(), text );

    return pm;
}

Load::Load( QWidget *parent, const char *name, WFlags f )
    : QWidget( parent, name, f )
{
    setBackgroundColor( black );
    points = 100;
    setMinimumWidth( points );
    userLoad = new double [points];
    systemLoad = new double [points];
    for ( int i = 0; i < points; i++ ) {
	userLoad[i] = 0.0;
	systemLoad[i] = 0.0;
    }
    maxLoad = 1.3;
    QTimer *timer = new QTimer( this );
    connect( timer, SIGNAL(timeout()), SLOT(timeout()) );
    timer->start( 2000 );
    gettimeofday( &last, 0 );
    first = TRUE;
}

void Load::paintEvent( QPaintEvent *ev )
{
    QPainter p( this );

    int mult = (int)(height() / maxLoad);

    p.setPen( gray );
    p.drawLine( 0, height() - mult, width(), height() - mult );
    p.drawText( 0, height() - mult, "100" );
    p.drawText( 0, height(), "0" );

    p.setPen( red );
    for ( int i = 1; i < points; i++ ) {
	int x1 = (i - 1) * width() / points;
	int x2 = i * width() / points;
	p.drawLine( x1, height() - userLoad[i-1] * mult,
		    x2, height() - userLoad[i] * mult );
    }

    p.setPen( green );
    for ( int i = 1; i < points; i++ ) {
	int x1 = (i - 1) * width() / points;
	int x2 = i * width() / points;
	p.drawLine( x1, height() - systemLoad[i-1] * mult,
		    x2, height() - systemLoad[i] * mult );
    }
}

void Load::timeout()
{
    int user;
    int usernice;
    int sys;
    int idle;
    FILE *fp;
    fp = fopen( "/proc/stat", "r" );
    fscanf( fp, "cpu %d %d %d %d", &user, &usernice, &sys, &idle );
    fclose( fp );
    struct timeval now;
    gettimeofday( &now, 0 );
    int tdiff = now.tv_usec - last.tv_usec;
    tdiff += (now.tv_sec - last.tv_sec) * 1000000;
    tdiff /= 10000;

    int udiff = user - lastUser;
    int sdiff = sys - lastSys;
    if ( tdiff > 0 ) {
	double uload = (double)udiff / (double)tdiff;
	double sload = (double)sdiff / (double)tdiff;
	if ( !first ) {
	    for ( int i = 1; i < points; i++ ) {
		userLoad[i-1] = userLoad[i];
		systemLoad[i-1] = systemLoad[i];
	    }
	    userLoad[points-1] = uload;
	    systemLoad[points-1] = sload;
//	    scroll( -width()/points, 0, QRect( 0, 0, width() - width()/points + 1, height() ) );
	    repaint( TRUE );
	    double ml = 1.3;
	    /*
	    for ( int i = 0; i < points; i++ ) {
		if ( userLoad[i] > ml )
		    ml = userLoad[i];
	    }
	    */
	    if ( maxLoad != ml ) {
		maxLoad = ml;
		update();
	    }
	}

	last = now;
	lastUser = user;
	lastSys = sys;
	first = FALSE;
    }
}

