/**********************************************************************
** Copyright (C) 2000 Trolltech AS.  All rights reserved.
**
** This file is part of Qt Palmtop Environment.
**
** This file may be distributed and/or modified under the terms of the
** GNU General Public License version 2 as published by the Free Software
** Foundation and appearing in the file LICENSE.GPL included in the
** packaging of this file.
**
** This file is provided AS IS with NO WARRANTY OF ANY KIND, INCLUDING THE
** WARRANTY OF DESIGN, MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE.
**
** See http://www.trolltech.com/gpl/ for GPL licensing information.
**
** Contact info@trolltech.com if any conditions of this licensing are
** not clear to you.
**
**********************************************************************/
#include "battery.h"
#include <qpainter.h>

#include <stdio.h>


BatteryMeter::BatteryMeter( QWidget *parent = 0 ) : QWidget( parent )
{
    startTimer( 5000 );
    setFixedHeight(12);
}

QSize BatteryMeter::sizeHint() const
{
    return QSize(6,12);
}

void BatteryMeter::timerEvent( QTimerEvent * )
{
    repaint(FALSE);
}

void BatteryMeter::paintEvent( QPaintEvent* )
{
    FILE *f = fopen("/proc/apm", "r");
    if ( f ) {
	QPainter p(this);
	int ac_status, bat_status, bat_flag, percent;
	// 1.13 1.2 0x02 0x00 0xff 0xff 49% 147 sec
	fscanf(f, "%*[^ ] %*d.%*d 0x%*x 0x%x 0x%x 0x%x %d%%",
		       &ac_status, &bat_status, &bat_flag, &percent);
	fclose(f);

	if ( percent < 0 ) {
	    if ( ac_status == 0x01 )
		percent = 100;
	    else
		percent = 5;
	} else if ( percent > 100 )
	    percent = 100;
	QColor c;
	if ( ac_status == 0x00 )
	    c = blue;
	else if ( ac_status == 0x01 )
	    c = green;
	else
	    c = red;
	p.setPen(QColor(80,80,80));
	p.drawLine(width()/4,0,width()/4+width()/2,0);
	p.drawRect(0,1,width(),height()-1);
	p.setBrush(c);
	int mpix = height()-3;
	int pix = percent * mpix / 100;
	p.fillRect(1,2+mpix-pix,width()-2,pix,c);
	if ( percent < 100 )
	    p.eraseRect(1,2,width()-2,QMAX(1,mpix-pix));
	int rpix = (percent - pix * 100 / mpix)*(mpix*(width()-2))/100;
	if ( rpix )
	    p.fillRect(1,2+mpix-pix-1,rpix,1,c);
    }
}
