/**********************************************************************
** Copyright (C) 2000 Trolltech AS.  All rights reserved.
**
** This file is part of Qt Palmtop Environment.
**
** This file may be distributed and/or modified under the terms of the
** GNU General Public License version 2 as published by the Free Software
** Foundation and appearing in the file LICENSE.GPL included in the
** packaging of this file.
**
** This file is provided AS IS with NO WARRANTY OF ANY KIND, INCLUDING THE
** WARRANTY OF DESIGN, MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE.
**
** See http://www.trolltech.com/gpl/ for GPL licensing information.
**
** Contact info@trolltech.com if any conditions of this licensing are
** not clear to you.
**
**********************************************************************/
#include <qpeapplication.h>
#include <qpopupmenu.h>
#include <qpushbutton.h>
#include <qtoolbutton.h>
#include <qwidget.h>
#include <qwindowsystem_qws.h>
#include <stdlib.h>
#include "inputmethods.h"
#include "common.h"


#ifdef _WS_QWS_
#define PEN_INPUT
#endif

#if !defined(QT_QWS_CASSIOPEIA)
#define UNICODE_INPUT
#define OPTI_INPUT
#endif

#ifdef PEN_INPUT
#include "inputpen.h"
#endif

#ifdef UNICODE_INPUT
#include "unikeyboard.h"
#endif
 
#include "keyboard.h"
#include "pickboard.h"


/* XPM */
static const char * tri_xpm[]={
"9 9 2 1",
"a c #000000",
". c None",
".........",
".........",
".........",
"....a....",
"...aaa...",
"..aaaaa..",
".aaaaaaa.",
".........",
"........."};


/* XPM */
static const char * pb_xpm[]={
"28 7 2 1",
"# c #303030",
"  c None",
" ########################## ",
" #   #   #   #   #   #    # ",
" #   #   #   #   #   #    # ",
" ########################## ",
" #     #   #   #   #      # ",
" #     #   #   #   #      # ",
" ########################## "};


/* XPM */
static const char * kb_xpm[]={
"28 13 2 1",
"# c #303030",
"  c None",
" ########################## ",
" #   #   #   #   #   #    # ",
" #   #   #   #   #   #    # ",
" ########################## ",
" #     #   #   #   #      # ",
" #     #   #   #   #      # ",
" ########################## ",
" #      #   #   #   #     # ",
" #      #   #   #   #     # ",
" ########################## ",
" #    #              #    # ",
" #    #              #    # ",
" ########################## "};


/* XPM */
static const char * opti_xpm[]={
"28 13 2 1",
"# c #303030",
"  c None",
" #########################  ",
" #   #   #   #   #   #   #  ",
" #   #   #   #   #   #   #  ",
" #########################  ",
" #   #   #   #   #   #   #  ",
" #   #   #   #   #   #   #  ",
" #########################  ",
" #   #   #   #   #   #   #  ",
" #   #   #   #   #   #   #  ",
" #########################  ",
" #   #   #   #   #   #   #  ",
" #   #   #   #   #   #   #  ",
" #########################  "};


/* XPM */
static const char * pen_xpm[] = {
"28 13 4 1",
"       c None",
".      c #000000",
"+      c #FFFFFF",
"@      c #808080",
"                            ",
"                         .  ",
"                        .+. ",
"                       ..@@.",
"                      .+@.. ",
"      ...            .+@@.  ",
"    .......         .+@@.   ",
"   ..     ..       .@.@.    ",
"   ..     ..       .@@.     ",
"   ..     ..      ....      ",
"  ..       ...  ....        ",
"            ......          ",
"                            "};


/* XPM */
static const char * uni_xpm[]={
"28 13 2 1",
"# c #000000",
". c None",
"............................",
"...####....#####.....####...",
"...####....######....####...",
"...####....#######..........",
"...####....########..####...",
"...####....####.####.####...",
"...####....####..########...",
"...####....####...#######...",
"...####....####....######...",
"...#####..#####.....#####...",
"....##########.......####...",
"......######..........###...",
"............................"};


static const int inputWidgetStyle = QWidget::WStyle_Customize | 
				    QWidget::WStyle_Tool |
				    QWidget::WStyle_StaysOnTop |
				    QWidget::WGroupLeader;


#ifdef PEN_INPUT
static QWidget *newPenInput()
{
    QWSPenInput *pi = new QWSPenInput( 0, 0, inputWidgetStyle );
    pi->setFrameStyle( QFrame::Box | QFrame::Plain );
    pi->setLineWidth( 1 );
    QString baseDir( getenv( "QTDIR" ) );
    baseDir += "/etc";
    pi->addCharSet( baseDir + "/qimpen/asciiupper.qpt" );
    pi->addCharSet( baseDir + "/qimpen/asciilower.qpt" );
    pi->addCharSet( baseDir + "/qimpen/numeric.qpt" );
    return pi;
}
#endif


static QWidget *newPickInput()
{
    Pickboard *pbd = new Pickboard( 0, 0, inputWidgetStyle );
    pbd->setFont( QFont( "smallsmooth", 9 ) );
    return pbd;
}


static QWidget *newKeyInput()
{
    Keyboard *kb = new Keyboard( 0, 0, inputWidgetStyle );
    kb->setMode( 0 );
    return kb;
}


static QWidget *newOptiInput()
{
    Keyboard *opti = new Keyboard( 0, 0, inputWidgetStyle );
    opti->setMode( 1 );
    return opti;
}


#ifdef UNICODE_INPUT
static QWidget *newUniInput()
{
    return new UniKeyboard( 0, 0, inputWidgetStyle );
}
#endif


struct {
    KeyMode keyMode;
    QWidget *inputObjectPtr;
    QWidget *(*newInputObject)( );
    const char **xpm;
    const char *menuName;
} inputMethods[] = {
    { Pick,    NULL, newPickInput, (const char **)pb_xpm,   "Pickboard"   },
#ifdef PEN_INPUT
    { Pen,     NULL, newPenInput,  (const char **)pen_xpm,  "Handwriting" },
#endif
    { Keybd,   NULL, newKeyInput,  (const char **)kb_xpm,   "Keyboard"    },
    { Opti,    NULL, newOptiInput, (const char **)opti_xpm, "Opti"        },
#ifdef UNICODE_INPUT
    { Unicode, NULL, newUniInput,  (const char **)uni_xpm,  "Unicode"     },
#endif
};


#undef USE_CONFIG_FILE


#ifdef USE_CONFIG_FILE
#define TASKBAR_OFFSET	    ((inputMethodsOverlapTaskBar) ? 0 : -5)
#define APPLICATION_OFFSET  ((inputMethodsOverlapApps) ? 0 : -5)
#else
#define TASKBAR_OFFSET	    (0)
#define APPLICATION_OFFSET  (0)
#endif


InputMethods::InputMethods( QWidget *parent ) : QWidget( parent, 0, WStyle_Tool | WStyle_Customize )
{
#ifdef USE_CONFIG_FILE
    QString *tmpBoolString;
    parseFileForArg( QPEApplication::qpeDir() + "apps/.qpe_menu.conf", "InputMethodsOverlapTaskBar", &tmpBoolString );
    inputMethodsOverlapTaskBar = (*tmpBoolString == "TRUE") ? TRUE : FALSE;
    parseFileForArg( QPEApplication::qpeDir() + "apps/.qpe_menu.conf", "InputMethodsOverlapApps", &tmpBoolString );
    inputMethodsOverlapApps = (*tmpBoolString == "TRUE") ? TRUE : FALSE;
    delete tmpBoolSting;
#endif

    keyboard = NULL;

    displayWidth  = qApp->desktop()->width();
    displayHeight = qApp->desktop()->height();

#ifdef _WS_QWS_
    QWSServer::setMaxWindowRect( QRect( 0, 0, displayWidth, displayHeight - 19 + TASKBAR_OFFSET ) );
#endif

    QHBoxLayout *hbox = new QHBoxLayout( this );

    kbdButton = new QPushButton( this );
    kbdButton->setFocusPolicy(NoFocus);
    kbdButton->setFlat(TRUE);
    kbdButton->setPixmap( QPixmap( (const char **)pb_xpm ) );
    kbdButton->setToggleButton( TRUE );
    kbdButton->setFixedHeight( 17 );
    hbox->addWidget( kbdButton );
    connect( kbdButton, SIGNAL(toggled(bool)), this, SLOT(showKbd(bool)) );

    kbdChoice = new QPushButton( this );
    kbdChoice->setFocusPolicy(NoFocus);
    kbdChoice->setPixmap( QPixmap( (const char **)tri_xpm ) );
    kbdChoice->setFlat(TRUE);
    kbdChoice->setFixedHeight( 17 );
    hbox->addWidget( kbdChoice );
    connect( kbdChoice, SIGNAL(clicked()), this, SLOT(chooseKbd()) );
}


void InputMethods::chooseKbd()
{
    QPopupMenu pop( this );

    for (unsigned i = 0; i < sizeof(inputMethods)/sizeof(inputMethods[0]); i++)
	pop.insertItem( inputMethods[i].menuName, i );

    pop.setItemChecked( keyMode, TRUE );

    QPoint pt = mapToGlobal(kbdChoice->geometry().topRight());
    QSize s = pop.sizeHint();
    pt.ry() -= s.height();
    pt.rx() -= s.width();
    int i = pop.exec( pt );
    if ( i == -1 )
	return;
    if ( i != keyMode && keyboard && keyboard->isVisible() )
	keyboard->hide();
    keyMode = (KeyMode)i;
    if ( !kbdButton->isOn() )
	kbdButton->setOn( TRUE );
    else
	showKbd( TRUE );
}


void InputMethods::showKbd( bool on )
{
    if ( on ) {
	// ensure keyMode is an index within the bounds of the array
	unsigned int j = ((unsigned int)keyMode < sizeof(inputMethods)/sizeof(inputMethods[0])) ? (unsigned int)keyMode : 0;
	keyMode = (KeyMode)j;

	if ( inputMethods[j].inputObjectPtr == NULL ) {
	    keyboard = inputMethods[j].inputObjectPtr = inputMethods[j].newInputObject();
	    int height = keyboard->sizeHint().height();
	    keyboard->resize( displayWidth, height );
	    keyboard->move( 0, mapToGlobal( QPoint() ).y() - height + TASKBAR_OFFSET );
	}

	keyboard = inputMethods[j].inputObjectPtr;
	keyboard->show();
	kbdButton->setPixmap( QPixmap( inputMethods[j].xpm ) );

#ifdef _WS_QWS_
        QWSServer::setMaxWindowRect( QRect( 0, 0, displayWidth, keyboard->y() + APPLICATION_OFFSET ) );
#endif

    } else {
	if ( keyboard )
	    keyboard->hide();
#ifdef _WS_QWS_
        QWSServer::setMaxWindowRect( QRect( 0, 0, displayWidth, mapToGlobal( QPoint() ).y() + TASKBAR_OFFSET ) );
#endif
    }
}


