/**********************************************************************
** Copyright (C) 2000 Trolltech AS.  All rights reserved.
**
** This file is part of Qt Palmtop Environment.
**
** This file may be distributed and/or modified under the terms of the
** GNU General Public License version 2 as published by the Free Software
** Foundation and appearing in the file LICENSE.GPL included in the
** packaging of this file.
**
** This file is provided AS IS with NO WARRANTY OF ANY KIND, INCLUDING THE
** WARRANTY OF DESIGN, MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE.
**
** See http://www.trolltech.com/gpl/ for GPL licensing information.
**
** Contact info@trolltech.com if any conditions of this licensing are
** not clear to you.
**
**********************************************************************/
#include "qspellchecker.h"
#include "global.h"
#include "qdawg.h"

#include <qfile.h>

QSpellChecker::QSpellChecker( QTextDocument *d )
    : QTextSyntaxHighlighter( d )
{
}

bool QSpellChecker::checkWord( const QString &word, int i, QTextParag *string )
{
    QTextFormat f;
    State s = findWordState( Global::fixedDawg().root(), word );
    if ( s == Invalid )
	s = findWordState( Global::addedDawg().root(), word );
    switch ( s ) {
    case Acceptable:
	f.setMisspelled( FALSE );
	string->setFormat( i - word.length(), word.length(), &f, TRUE, QTextFormat::Misspelled );
	break;
    case Intermediate:
	f.setMisspelled( FALSE );
	string->setFormat( i - word.length(), word.length(), &f, TRUE, QTextFormat::Misspelled );
	break;
    case Invalid:
	f.setMisspelled( TRUE );
	string->setFormat( i - word.length(), word.length(), &f, TRUE, QTextFormat::Misspelled );
	return FALSE;
	break;
    }
    return TRUE;
}

void QSpellChecker::highlighte( QTextParag *string, int, bool )
{
    QString s = string->string()->toString();
    QChar c;
    QString lastWord;

    for ( int i = 0; i < (int)s.length(); ++i ) {
	c = s[ i ];
	if ( c.isPunct() || c.isSpace() || c.isMark() || c.isNumber() || c.isDigit() || c.isNull() ) {
	    if ( lastWord.isEmpty() ) {
		lastWord = QString::null;
		continue;
	    }
	    if ( !checkWord( lastWord, i, string ) )
		checkWord( lastWord.lower(), i, string );
	    lastWord = QString::null;
	} else {
	    lastWord += c;
	}
    }

    string->setFirstHighlighte( FALSE );
    string->setEndState( 0 );
}

QSpellChecker::State QSpellChecker::findWordState(const QDawg::Node* n, const QString& s, int index ) const
{
    if ( s[index] == ' ' ) {
	State st = findWordState(Global::fixedDawg().root(), s, index+1);
	if ( st == Invalid )
	    st = findWordState( Global::addedDawg().root(), s, index+1);
	return st;
    }
    if ( n ) {
	if ( s[index] == n->letter() ) {
	    if ( index == (int)s.length()-1 ) {
		return n->isWord() ? Acceptable : Intermediate;
	    }
	    return findWordState(n->jump(), s, index+1);
	} else {
	    return findWordState(n->next(), s, index);
	}
    }
    return index == 0 || s[index-1] == ' ' ? Acceptable : Invalid;
}
